from ._api import asserto
from ._api import register_assert
from ._asserto import Asserto
from ._exceptions import UnsupportedHandlerTypeError
from ._warnings import NoAssertAttemptedWarning

__all__ = ("asserto", "register_assert", "Asserto", "NoAssertAttemptedWarning", "UnsupportedHandlerTypeError")



import functools
import typing


def update_triggered(fn: typing.Callable[[typing.Any], typing.Any]) -> typing.Callable[[typing.Any], typing.Any]:
    """
    Track the triggered state on any asserto calls to ensure no instances were created
    and used without calling any assertable methods.
    :param fn: The asserting function
    """

    @functools.wraps(fn)
    def wrapper(*args, **kwargs) -> typing.Callable[[typing.Any], typing.Any]:
        instance = args[0]
        instance.triggered = True
        return fn(*args, **kwargs)

    return wrapper



import typing

from ..descriptors import ValidatesInstanceOf
from ._handler import Handler


class StringHandler(Handler):
    """
    A handler responsible for all string based checks.
    """

    actual: str = ValidatesInstanceOf(str)  # type: ignore[assignment]

    def __init__(self, actual: typing.Any) -> None:
        super().__init__(actual)

    def is_alpha(self) -> None:
        if self.raise_if_length_equals() and not self.actual.isalpha():
            raise AssertionError(f"{self.actual} did not contain only alpha numeric characters.")

    def is_digit(self) -> None:
        if self.raise_if_length_equals() and not self.actual.isdigit():
            raise AssertionError(f"{self.actual} did not contain only numeric digits.")

    def ends_with(self, suffix: str) -> None:
        if not self.actual.endswith(suffix):
            raise AssertionError(f"{self.actual} did not end with {suffix=}")

    def starts_with(self, prefix: str) -> None:
        if not self.actual.startswith(prefix):
            raise AssertionError(f"{self.actual} did not begin with {prefix=}")
